package com.dcc.account;

import java.io.Serializable;
import java.math.BigDecimal;

/**
 * <p>A mutable class responsible for containing the confirmation 
 * details for a successful credit card transaction.</p>
 * <p>
 * This class is mutuable only to support the JavaBean specification,
 * the webservice tools require that this class have a default
 * constructor and set methods for every field.
 * </p>
 * 
 */
public final class TransactionConfirmation implements Serializable {

	private BigDecimal	amount;
	private String		txDate;
	private String		creditCardNumber;
	private String		txDesc;
	
	/**
	 * Default Constructor
	 */
	public TransactionConfirmation() { }
	
	/**
	 * Constructor for TransactionRecord
	 */
	public TransactionConfirmation(BigDecimal amount, String txDate, String creditCardNumber, String txDesc) {
		this.amount = amount;
		this.txDate = txDate;
		this.creditCardNumber = creditCardNumber;
		this.txDesc = txDesc;
	}

	/**
	 * @return Returns the transaction amount
	 */
	public BigDecimal getAmount() {
		return amount;
	}

	/**
	 * @return Returns the transaction Date
	 */
	public String getTxDate() {
		return txDate;
	}

	/**
	 * @return Returns the credit card's number
	 */
	public String getCreditCardNumber() {
		return creditCardNumber;
	}

	/**
	 * @return Returns the transaction description
	 */
	public String getTxDesc() {
		return txDesc;
	}
	/**
	 * Sets the amount.
	 * @param amount The amount to set
	 */
	public void setAmount(BigDecimal amount) {
		this.amount = amount;
	}

	/**
	 * Sets the credit card number.
	 * @param creditCardNumber The creditCardNumber to set
	 */
	public void setCreditCardNumber(String creditCardNumber) {
		this.creditCardNumber = creditCardNumber;
	}

	/**
	 * Sets the transaction date.
	 * @param txDate The date to set
	 */
	public void setTxDate(String txDate) {
		this.txDate = txDate;
	}

	/**
	 * Sets the Description
	 * @param txDesc The description to set
	 */
	public void setTxDesc(String txDesc) {
		this.txDesc = txDesc;
	}


	public String toString() {
		return "Transaction confirmed for the purchase of a " + txDesc + " for an amount of " + amount + 
					" on  " + txDate + " using Dream Credit Card " + creditCardNumber;
	}


}
