package com.ibm.oiddemo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.ejb.CreateException;
import javax.ejb.FinderException;

import org.eclipse.emf.ecore.EObject;

import com.ibm.etools.service.locator.ServiceLocatorManager;
import com.ibm.oid.util.OIDInfo;
import com.ibm.oid.util.OIDTypeMap;

/**
 * Bean implementation class for Enterprise Bean: OIDGeneratorSession
 */
public class OIDGeneratorSessionBean implements javax.ejb.SessionBean {
	private javax.ejb.SessionContext mySessionCtx;
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}

	private static String STATIC_OIDvalueHome_REF_NAME = "ejb/oidValues";
	private static Class STATIC_OIDvalueHome_CLASS = OIDvalueHome.class;

	protected static final synchronized OIDvalueHome getOIDHome() {

		return (OIDvalueHome) ServiceLocatorManager.getLocalHome(STATIC_OIDvalueHome_REF_NAME, STATIC_OIDvalueHome_CLASS);
	}

	/**
	 * getNextOID(String type)
	 * 		return the next available IntegerOID for the specified type name. 
	 * @param type
	 * @return
	 */
	public Integer getNextOID(String type) {
		OIDvalue existingOid = getOIDvalue(type);
		return existingOid == null ? new Integer(-1) : existingOid.increment();
	}
	
	private OIDvalue getOIDvalue(String type) {
		OIDvalue oidValue = null;
		try {
			oidValue = getOIDHome().findByPrimaryKey(type);
		} catch (FinderException e) {
			// Bootstrapping case, create the new OID value
			try {
				oidValue = getOIDHome().create(type);
			} catch (CreateException ee) {
				log("Create Exception creating OID for type: " + type + "; error: " + ee);
			}
		}
		return oidValue;
	}
	
	/**
	 * Return the last available Integer OID for the specified type name and block.
	 * This means that the returned Integer and OIDs prior to this one  
	 * @param type
	 * @return
	 */
	public Integer getNextOIDs(String type, int block) {
		OIDvalue existingOid = getOIDvalue(type);
		return existingOid == null ? new Integer(-1) : existingOid.increment(block);
	}

	/**
	 * getNextOID(String type)
	 * 		return the next available OID for the specified type name as a String. 
	 * @param type
	 * @return 
	 */
	public String getNextOIDString(String type) {
		return getNextOID(type).toString();
	}

	/**
	 * Method log.
	 * @param string
	 */
	private void log(String string) {
		System.out.println(string);
	}
	
	public void setAutoIDsIfNecessary(EObject eObject, OIDTypeMap typeMap) {
		if (eObject == null || typeMap == null) return;
		Map grouped = groupTypesNeedingIDs(eObject, typeMap);
		if (grouped != null && !grouped.isEmpty()) {
			setAutoIDs(grouped);
		}
	}
	/**
	 * @param grouped
	 */
	private void setAutoIDs(Map grouped) {
		if (grouped == null) return;
		Iterator it = grouped.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
			setAutoIDs((OIDInfo) entry.getKey(), (List) entry.getValue());
		}
		
	}
	/**
	 * @param info
	 * @param list
	 */
	private void setAutoIDs(OIDInfo info, List list) {
		Integer lastOIDInt = getNextOIDs(info.getTypeName(), list.size());
		int lastOID = lastOIDInt.intValue();
		for (int i = 0; i < list.size(); i++) {
			EObject eObject = (EObject) list.get(i);
			info.setIDValue(eObject, lastOID);
			lastOID--;
		}
	}
	/**
	 * @param eObject
	 * @param typeMap
	 * @return
	 */
	private Map groupTypesNeedingIDs(EObject eObject, OIDTypeMap typeMap) {
		Iterator it = eObject.eAllContents();
		Map grouped = null;
		EObject content = eObject;
		boolean isFirst = true;
		while (isFirst || it.hasNext()) {
			if (!isFirst)
				content = (EObject) it.next();
			else
				isFirst = false;
			OIDInfo info = (OIDInfo) typeMap.get(content.eClass());
			if (info == null) continue;
			if (info.needsID(content)) {
				if (grouped == null)
					grouped = new HashMap();
				List needsID = getOrCreateIDList(grouped, info);
				if (needsID != null)
					needsID.add(content);
			}
		}
		return grouped;
	}
	/**
	 * @param grouped
	 * @param info
	 * @return
	 */
	private List getOrCreateIDList(Map grouped, OIDInfo info) {
		List needsIDs = (List) grouped.get(info);
		if (needsIDs == null) {
			needsIDs = new ArrayList();
			if (grouped != null)
				grouped.put(info, needsIDs);
		}
		return needsIDs;
	}

}
