/*
 */
package com.ibm.oid.util;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 */
public class OIDInfo {
	private String typeName;
	private EAttribute idAttribute;
	private EClass eClass;
	
	OIDInfo(EClass eClass, String typeName, EAttribute idAttribute) {
		this.eClass = eClass;
		this.typeName = typeName;
		this.idAttribute = idAttribute;
	}
	OIDInfo(EClass eClass, String typeName, String idAttributeName) {
		this.eClass = eClass;
		this.typeName = typeName;
		initializeIDAttribute(idAttributeName);
	}
	/**
	 * @param idAttributeName
	 */
	private void initializeIDAttribute(String idAttributeName) {
		EStructuralFeature feature = eClass.getEStructuralFeature(idAttributeName);
		if (feature == null || !(feature instanceof EAttribute))
			throw new IllegalStateException("No id attribute named " + idAttributeName + " for type " + eClass.getName());
		idAttribute = (EAttribute) feature;		
	}
	
	public void setIDValue(EObject eObject, int value) {
		if (eObject == null || eObject.eClass() != eClass)
			throw new IllegalStateException("Cannot set id value for an ivalid object " + eObject);
		eObject.eSet(idAttribute, new Integer(value));
	}
	
	public boolean needsID(EObject eObject) {
		if (eObject == null || eObject.eClass() != eClass)
			throw new IllegalStateException("Cannot set id value for an ivalid object " + eObject);
		return !eObject.eIsSet(idAttribute);
	}
	/**
	 * @return Returns the typeName.
	 */
	public String getTypeName() {
		return typeName;
	}	
}
